/**
 ** Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

/**
 **  gcspy_comm.cpp
 **
 **  Abstractions over sockets
 **/

#include <stdio.h>

#include "gcspy_timer.hpp"
#include "gcspy_array_input.hpp"
#include "gcspy_array_output.hpp"

#include "gcspy_comm.hpp"

// SERVER STUFF

gcspyServer::gcspyServer (int port, int max_backlog) {
  _fd = socket(AF_INET, SOCK_STREAM, 0);
  if (_fd < 0)
    gcspy_raise_error("could not create socket");

  int x = 1;
  setsockopt(_fd, SOL_SOCKET,SO_REUSEADDR, &x, sizeof(int));

  _addr.sin_family = AF_INET;
  _addr.sin_addr.s_addr = htonl(INADDR_ANY);
  _addr.sin_port = htons(port);

  int res = bind(_fd,
		 (struct sockaddr *) &_addr,
		 sizeof(_addr));
  if (res < 0)
    gcspy_raise_error("bind failed");

  listen(_fd, max_backlog);
}

int
gcspyServer::accept (struct sockaddr *client_addr, unsigned int *addr_len) {
  int fd = ::accept(_fd, client_addr, addr_len);
  if (fd < 0)
    gcspy_raise_error("accept failed");
  return fd;
}

void
gcspyServer::close () {
  gcspyTimer::wait(100);
  ::close(_fd);
}

// CLIENT STUFF

gcspyClient::gcspyClient () {
  _in_buffer = NULL;
  _tmp_buffer = NULL;
  _out_buffer = NULL;
}

gcspy_length_t
gcspyClient::read_stream_len () {
  return (gcspy_length_t) gcspyArrayInput::read_int(_tmp_buffer);
}

void
gcspyClient::write_stream_len (gcspy_length_t len) {
  char buffer[prefix_size];
  gcspyArrayOutput::write_int(buffer, (int) len);
  int res = ::send(_fd, buffer, prefix_size, 0);
  if (res < prefix_size)
    gcspy_raise_error("send failed");
}

void
gcspyClient::wait_for_connection (gcspyServer *server,
				  gcspy_length_t max_len) {
  unsigned int addr_len = sizeof(_addr);
  _fd = server->accept((struct sockaddr *) &_addr, &addr_len);
  _max_len = max_len;

  gcspy_assert(_in_buffer == NULL);
  _in_buffer = new char[max_len];
  if (_in_buffer == NULL) {
    char buffer[128];
    sprintf(buffer, "new failed for a buffer of len %u", max_len);
    gcspy_raise_error(buffer);
  }

  _tmp_len = 0;
  gcspy_assert(_tmp_buffer == NULL);
  _tmp_buffer = new char[2*max_len];
  if (_tmp_buffer == NULL) {
    char buffer[128];
    sprintf(buffer, "new failed for a buffer of len %u", 2*max_len);
    gcspy_raise_error(buffer);
  }

  gcspy_assert(_out_buffer == NULL);
  _out_buffer = new char[max_len];
  if (_out_buffer == NULL) {
    char buffer[128];
    sprintf(buffer, "new failed for a buffer of len %u", max_len);
    gcspy_raise_error(buffer);
  }
}

void
gcspyClient::receive () {
  gcspy_length_t total_len;

  while (_tmp_len < prefix_size) {
    int res = recv(_fd, _tmp_buffer + _tmp_len, _max_len, 0);
    if (res < 0)
      gcspy_raise_error("receive failed");
    if (res == 0) {
      _len = 0;
      _tmp_len = 0;
      return;
    }
    _tmp_len += res;
  }
  _len = read_stream_len();

  total_len = _len + prefix_size;
  while (_tmp_len < total_len) {
    int res = recv(_fd, _tmp_buffer + _tmp_len, _max_len, 0);
    if (res <= 0)
      gcspy_raise_error("receive failed");
    _tmp_len += res;
  }

  memcpy(_in_buffer, _tmp_buffer + prefix_size, _len);
  if (_tmp_len > total_len) {
    memcpy(_tmp_buffer, _tmp_buffer + total_len, _tmp_len - total_len);
    _tmp_len -= total_len;
  } else {
    _tmp_len = 0;
  }
}

gcspy_bool_t
gcspyClient::definitely_receive () {
  receive();
  if (has_finished())
    return 0;
  return 1;
}

void
gcspyClient::send (gcspy_length_t len) {
  write_stream_len(len);
  int res = ::send(_fd, _out_buffer, len, 0);
  if (res != len)
    gcspy_raise_error("send failed");
}

void
gcspyClient::close () {
  gcspyTimer::wait(100);

  delete _in_buffer;
  _in_buffer = NULL;

  delete _out_buffer;
  _out_buffer = NULL;

  delete _tmp_buffer;
  _tmp_buffer = NULL;

  ::close(_fd);
}
