/**
 ** $Id: TerminalStoreTrace.java 21 2005-06-11 00:25:23Z rej $
 ** Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

package gcspy.tools;

import gcspy.interpreter.TraceData;
import gcspy.interpreter.client.ClientInterpreter;
import gcspy.utils.Arguments;
import gcspy.utils.FileUtils;
import gcspy.utils.Timer;
import gcspy.utils.Utils;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Dumps the transmission from the server to a file
 * 
 * @author Tony Printezis
 */
public class TerminalStoreTrace extends OutputStreamTrace {
	
	private static final Logger logger = Logger.getLogger(TerminalStoreTrace.class.getName());

	private ClientInterpreter interpreter;
	private TraceData data;
	
	protected TraceData getData() {
		return data;
	}
	
	private static String filename;
	
	private void amendGeneralInfo(String host, int port) {
		String info = getData().getGeneralInfo();
		String amended = generalInfo() + info;
		getData().setGeneralInfo(amended);
	}

	private void go() throws IOException {
		Timer timer = new Timer();

		if (logger.isLoggable(Level.INFO)) {
			logger.log(Level.INFO, "Output file: " + filename + ", compression type: " + getCompressionType());
			if (getMaxCount() > 0)
				logger.log(Level.INFO, "Max events: " + getMaxCount());
		}

		logger.log(Level.FINE, "Writing bootstrap information");
		getData().serialiseEverything(this);

		interpreter.addEventListener(this);
		interpreter.addSpaceListener(this);

		timer.start();
		interpreter.mainLoop();
		timer.stop();

		close();

		long length = FileUtils.length(filename);
		double kbs = (double) length / 1024;
		double mbs = kbs / 1024;
		if (logger.isLoggable(Level.INFO)) {
			logger.log(Level.INFO, "Summary:  Wrote " + Utils.formatSize(mbs) + " MBs (" + Utils.formatSize(kbs)
					+ " KBs, " + Utils.formatSize(length) + " bytes), wrote " + getCount()
					+ " events, storing time was " + (timer.getTime() / 1000) + " secs");
		}
	}
	
	protected ClientInterpreter getInterpreter() { return interpreter; }

	/******************** Constructor ********************/

	private TerminalStoreTrace(String host, int port, 
			String fileName, int count)
			throws IOException {
		super(new FileOutputStream(fileName), count, compressionFor(fileName));

		data = new TraceData();
		interpreter = new ClientInterpreter(data);

		logger.log(Level.INFO, "Connecting to " + host + ":" + port);
		interpreter.connectToServer(host, port, false);

		amendGeneralInfo(host, port);
	}

	/******************** Main ********************/

	static private void usage(String mesg) {
		System.out.println();
		System.out.println("usage:");
		System.out.println("    gcspy.tools.TerminalStoreTrace <options>");
		System.out.println();
		System.out.println("where <options> are");
		System.out.println("    -server <host> <port>");
		System.out.println("    -output <file name>");
		System.out.println("    -count <count> (optional)");
		System.out.println("    -overwrite (optional)");
		System.out.println("    -verbose (optional)");
		if (mesg != null) {
			reportError(mesg);
		}
		System.exit(-1);
	}

	static private void reportError(String mesg) {
		logger.log(Level.SEVERE, mesg);
	}

	static public void main(String args[]) {
		TerminalStoreTrace storeTrace;
		Arguments arguments = new Arguments();

		int serverParams[] = { Arguments.STRING_TYPE, Arguments.POS_INT_TYPE };
		arguments.add("-server", true, serverParams);

		int fileParam[] = { Arguments.STRING_TYPE };
		arguments.add("-output", true, fileParam);

		arguments.add("-overwrite");

		int countParam[] = { Arguments.POS_INT_TYPE };
		Object countDefault[] = { -1 };
		arguments.add("-count", false, countParam, countDefault);

		arguments.add("-verbose");

		if (!arguments.parse(args)) {
			usage(arguments.getError());
		}

		Object values[];

		values = arguments.getValues("-server");
		String host = (String) values[0];
		int port = (Integer) values[1];

		values = arguments.getValues("-output");
		filename = (String) values[0];

		boolean overwrite = arguments.isSet("-overwrite");

		values = arguments.getValues("-count");
		int count = (Integer) values[0];

		boolean verbose = arguments.isSet("-verbose");
		if (verbose)
			Logger.getLogger(Logger.GLOBAL_LOGGER_NAME).setLevel(Level.ALL);

		try {
			if (overwrite) {
				File file = new File(filename);
				file.delete();
			}
			storeTrace = new TerminalStoreTrace(host, port, filename, count);
			storeTrace.go();
		} catch (IOException e) {
			logger.log(Level.SEVERE, "Exception caught, terminating", e);
			System.exit(-1);
		}
	}

}
