package gcspy.vis;

import gcspy.interpreter.DataAccessor;
import gcspy.interpreter.Space;
import gcspy.interpreter.Stream;
import gcspy.vis.TileStyle.ValuePaintStyle;
import gcspy.vis.utils.AdjustedColor;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.geom.Rectangle2D;
import java.util.EnumSet;

import javax.swing.JComponent;

/**
 * Component that displays blocks of a space (tiles).
 */
class TileComponent extends JComponent {

	/**
	 * Tile types.
	 */
	public enum TileType {
		VALUE, OVERFLOW, UNUSED, ILLEGAL,
	}

	/**
	 * Tile flags.
	 */
	public enum TileFlags {
		SEPARATOR_LEFT,  // tile with separator on the left
		SEPARATOR_RIGHT, // tile with separator on the right
		LINK_LEFT,       // tile with link on the left
		LINK_RIGHT       // tile with link on the right
	}

	/**
	 * Pattern that displays a vertical bar for which the height depends on a
	 * specified value and the maximum value possible.
	 * 
	 * @author Peter Hofer
	 * 
	 */
	private static class VBarPattern implements FillPattern {
		private int value;
		private int max;

		public VBarPattern(int value, int max) {
			this.value = value;
			this.max = max;
		}

		@Override
		public void fill(Graphics g, Rectangle bounds) {
			int height = (bounds.height * value / max);
			int y = bounds.y + bounds.height - height;
			g.fillRect(bounds.x, y, bounds.width, height);
		}
	}

	/**
	 * Pattern that is drawn to indicate an illegal value for a stream.
	 * 
	 * @author Peter Hofer
	 */
	private static class IllegalPattern implements FillPattern {
		public static final Color COLOR = Color.RED;

		@Override
		public void fill(Graphics g, Rectangle area) {
			g.fillRect(area.x, area.y, area.width, area.height);

			// Centered text
			final String text = "?";
			Rectangle2D textBounds = g.getFontMetrics().getStringBounds(text, g);
			g.setColor(Color.WHITE);
			g.drawString(text, (int) (area.getCenterX() - textBounds.getWidth() / 2),
				(int) (area.getCenterY() + (-textBounds.getY() / 2)));
		}
	}

	private static FillPattern ILLEGAL_PATTERN = new IllegalPattern();

	private TileStyle style;

	private EnumSet<TileFlags> tileFlags;
	private boolean selected = false;
	private boolean marked = false;

	private Color[] colors;
	private FillPattern[] patterns;

	/**
	 * Constructor.
	 * 
	 * @param style
	 *            Color scheme to use.
	 */
	public TileComponent(TileStyle style) {
		setOpaque(true);
		setPreferredSize(TileManager.TILE_SIZE_DEFAULT);
		setColorConfig(style);

		// May be painted before any data is available, so use
		// reasonable presets for color and border
		tileFlags = EnumSet.noneOf(TileFlags.class);
		patterns = new FillPattern[] { FillPattern.OPAQUE };
		colors = new Color[] { style.getUnusedColor() };
	}

	/**
	 * Sets a different color scheme.
	 * 
	 * @param colors
	 *            New color scheme. Must not be <code>null</code>.
	 */
	public void setColorConfig(TileStyle colors) {
		if (colors == null)
			throw new NullPointerException("colors");

		this.style = colors;
	}

	/**
	 * Sets whether this tile is displayed as being selected or not.
	 * 
	 * @param selected
	 *            Select status of this tile.
	 */
	public void setSelected(boolean selected) {
		this.selected = selected;
	}

	/**
	 * Sets whether this tile is displayed as being marked or not.
	 * 
	 * @param selected
	 *            Marking status of this tile.
	 */
	public void setMarked(boolean marked) {
		this.marked = marked;
	}

	/**
	 * Changes the appearance of this component to reflect the values of the
	 * specified {@link Tile} object.
	 */
	public void setTile(Tile tile) {
		setBorder(null);

		if (tile != null) {
			Space space = tile.getSpace();
			int[] streamIds = tile.getStreams();

			TileType[] types = new TileType[streamIds.length];
			EnumSet<TileFlags> flags = EnumSet.noneOf(TileFlags.class);
			int[] values = new int[streamIds.length];

			for (int i = 0; i < streamIds.length; ++i) {
				Stream stream = space.getStream(streamIds[i]);
				int index = tile.getIndex();

				DataAccessor accessor = stream.getAccessor();
				if (!accessor.isDataAvailable())
					throw new RuntimeException("No data for stream " + streamIds[i]);
				
				byte[] controls = space.getControl();
				byte control = controls[index];
				if (Space.isControlUsed(control)) {
					if (accessor.isOverflow(index)) {
						types[i] = TileType.OVERFLOW;
					} else {
						types[i] = TileType.VALUE;
					}
				} else if (Space.isControlUnused(control)) {
					types[i] = TileType.UNUSED;
				}

				if (Space.isControlSeparator(control)) {
					flags.add(TileFlags.SEPARATOR_LEFT);
				}
				int succIndex = index + 1;
				if (succIndex < controls.length) {
					byte succ = controls[succIndex];
					if (Space.isControlSeparator(succ)) {
						flags.add(TileFlags.SEPARATOR_RIGHT);
					}
				}

				if (Space.isControlLink(control)) {
					flags.add(TileFlags.LINK_RIGHT);
				}
				int prevIndex = index - 1;
				if (prevIndex >= 0) {
					byte prev = controls[prevIndex];
					if (Space.isControlLink(prev)) {
						flags.add(TileFlags.LINK_LEFT);
					}
				}

				if (stream.getPresentation() == Stream.PRESENTATION_ENUM) {
					values[i] = accessor.get(index);
				} else {
					if (types[i] == TileType.VALUE || types[i] == TileType.OVERFLOW) {
						values[i] = accessor.getAdjusted(index);
					}
				}
			}

			setTile(space, streamIds, types, flags, values);
		} else {
			setOpaque(false);
		}
	}

	/**
	 * Changes the appearance of this component to reflect the specified values.
	 */
	public void setTile(Space space, int[] streamIds, TileType[] types, EnumSet<TileFlags> flags, int[] values) {
		if (streamIds.length != types.length || types.length != values.length)
			throw new IllegalArgumentException("Different number of stream ids, types or values");

		colors = new Color[streamIds.length];
		patterns = new FillPattern[streamIds.length];

		if (flags == null) {
			flags = EnumSet.noneOf(TileFlags.class);
		}
		tileFlags = flags;

		for (int i = 0; i < streamIds.length; ++i) {
			int streamId = streamIds[i];
			
			patterns[i] = FillPattern.OPAQUE; // Preset for all streams
			switch (types[i]) {
			case VALUE:
			case OVERFLOW:
				Stream stream = space.getStream(streamId);
				int value = values[i];
				if (!stream.isWithinBounds(value)) {
					// Value is outside of valid bounds: render tile accordingly
					colors[i] = IllegalPattern.COLOR;
					patterns[i] = ILLEGAL_PATTERN;
				} else if (stream.getPresentation() == Stream.PRESENTATION_ENUM) {
					switch (style.getEnumPaintStyle()) {
					case COLORED_PATTERNS:
						patterns[i] = TileStyle.getEnumerationPattern(value);
						colors[i] = TileStyle.getEnumerationPatternColor(value);
						break;

					case MONOCHROME_PATTERNS:
						colors[i] = Color.BLACK;
						patterns[i] = TileStyle.getEnumerationPattern(value);
						break;

					case COLORED_OPAQUE:
						colors[i] = TileStyle.getEnumerationColor(value);
						break;
					}
				} else {
					DataAccessor accessor = stream.getAccessor();
					AdjustedColor adjcolor = new AdjustedColor(style.getStreamColor(streamId), accessor.getAdjustedMax());
					colors[i] = adjcolor.generate(value);

					if (style.getValuePaintStyle() == ValuePaintStyle.VERTICAL_BAR) {
						patterns[i] = new VBarPattern(value, accessor.getAdjustedMax());
					}
				}
				break;

			case UNUSED:
				colors[i] = style.getUnusedColor();
				break;

			case ILLEGAL:
				colors[i] = IllegalPattern.COLOR;
				patterns[i] = ILLEGAL_PATTERN;
			}
		}
	}

	public void setTile(Space space, int streamId, TileType type, EnumSet<TileFlags> flags, int value) {
		setTile(space, new int[] { streamId }, new TileType[] { type }, flags, new int[] { value });
	}

	@Override
	protected void paintComponent(Graphics g) {
		Rectangle canvas = new Rectangle(getSize());

		// fill with background
		if (marked) {
			g.setColor(style.getMarkerColor());
		} else {
			g.setColor(getBackground());
		}
		g.fillRect(canvas.x, canvas.y, canvas.width, canvas.height);

		// padding on top and bottom
		canvas.y++;
		canvas.height -= 2;

		if (tileFlags.contains(TileFlags.SEPARATOR_LEFT)) {
			// insert separator and padding on the left
			g.setColor(style.getSeparatorColor());
			g.drawLine(canvas.x, canvas.y, canvas.x, canvas.height);
			canvas.x += 3;
			canvas.width -= 3;
		} else if (tileFlags.contains(TileFlags.LINK_LEFT)) {
			// display a link to the previous tile on the left
			final int y = canvas.y + (canvas.height / 2);
			g.setColor(style.getLinkColor());
			g.drawLine(canvas.x, y, canvas.x + 1, y);
			canvas.x += 2;
			canvas.width -= 2;
		} else {
			// insert padding on the left
			canvas.x++;
			canvas.width--;
		}

		if (tileFlags.contains(TileFlags.SEPARATOR_RIGHT)) {
			// insert padding and a separator on the right
			g.setColor(style.getSeparatorColor());
			g.drawLine(canvas.x + canvas.width - 1, canvas.y, canvas.x + canvas.width - 1, canvas.height);
			canvas.width -= 3;
		} else if (tileFlags.contains(TileFlags.LINK_RIGHT)) {
			// display a link to the next tile on the right
			final int y = canvas.y + (canvas.height / 2);
			g.setColor(style.getLinkColor());
			g.drawLine(canvas.x + canvas.width - 2, y, canvas.x + canvas.width - 1, y);
			canvas.width -= 2;
		} else {
			// insert padding on the right
			canvas.width--;
		}

		// draw a raised border
		int thickness = 1;
		Color border = Color.DARK_GRAY;
		if (selected) {
			border = style.getSelectedColor();
			thickness = 2;
		}
		g.setColor(border);
		g.fill3DRect(canvas.x, canvas.y, canvas.width, canvas.height, true);
		canvas.x += thickness;
		canvas.y += thickness;
		canvas.width -= (2 * thickness);
		canvas.height -= (2 * thickness);

		// background for actual graphic
		g.setColor(getBackground());
		g.fillRect(canvas.x, canvas.y, canvas.width, canvas.height);

		// split vertically into sections for each displayed stream
		Rectangle section = new Rectangle(canvas);
		section.width = section.width / patterns.length;
		for (int i = 0; i < patterns.length; ++i) {
			if (i == patterns.length - 1) {
				// right-most section gets remaining width of canvas
				// (takes care of rounding error in section width)
				section.width = canvas.x + canvas.width - section.x;
			}

			// draw stream visualization
			g.setClip(section);
			g.setColor(colors[i]);
			patterns[i].fill(g, section);

			if (i > 0) {
				// paint divider line on the left
				g.setColor(border);
				g.drawLine(section.x, section.y, section.x, section.y + section.height);
			}

			section.x += section.width;
		}
	}
}
