package gcspy.vis;

import gcspy.interpreter.Events;
import gcspy.interpreter.Space;
import gcspy.interpreter.TraceData;
import gcspy.interpreter.client.EventCommandListener;
import gcspy.interpreter.client.PauseListener;
import gcspy.interpreter.client.SpaceCommandListener;

import java.util.ArrayList;
import java.util.List;

import javax.swing.SwingUtilities;

public class UIInterpreterProxy implements EventCommandListener, PauseListener, SpaceCommandListener, InterpreterEventProvider {
	private TraceData data;
	
	private TraceData getData() {
		return this.data;
	}
	
	// Listeners
	private List<PauseListener> pauseListeners;

	private boolean listenersEnabled;
	
	private boolean synchronousOperation;
	
	public boolean isSynchronous() {
		return this.synchronousOperation;
	}
	
	public void setSynchronous(boolean newval) {
		this.synchronousOperation = newval;
	}
	
	public boolean isListenersEnabled() {
		return this.listenersEnabled;
	}

	public void setListenersEnabled(boolean listenersEnabled) {
		this.listenersEnabled = listenersEnabled;
	}

	private List<EventCommandListener> eventListeners;
	private List<SpaceCommandListener> spaceListeners;

	@Override
	public void onEventCmd(int eventID, int elapsedTime, int compensationTime) {
		if (isListenersEnabled() && isSynchronous()) {
			fireEventListeners(eventID, elapsedTime, compensationTime);
		}
	}
	
	/** ****************** Utilities ******************* */

	/**
	 * Calculate the maximum value in each stream that uses a
	 * <code>PRESENTATION_MAX_VAR</code> presentation style
	 */
	private void calcSpaceMaxima(Space space) {
		for (int i = 0; i < space.getStreamNum(); ++i) {
			space.getStream(i).calcMaxIfNecessary();
		}
	}	
	
	private void calcMaxima() {
		for (int i = 0; i < getData().getSpaceNum(); ++i) {
			calcSpaceMaxima(getData().getSpace(i));
		}
	}	

	public synchronized void fireEventListeners(final int eventID, final int elapsedTime,
			final int compensationTime) {
		assert isListenersEnabled();
		calcMaxima();

		for (final EventCommandListener l : eventListeners)
		{
			assert l != null;
			SwingUtilities.invokeLater(new Runnable()
			{
				@Override
				public void run()
				{
					l.onEventCmd(eventID, elapsedTime, compensationTime);
				}
			});
		}
	}

	@Override
	public void onPause() {
		if (isListenersEnabled()) {
			firePauseListeners();
		}
	}

	public synchronized void firePauseListeners() {
		assert isListenersEnabled();
		for (final PauseListener l : pauseListeners)
		{
			assert l != null;
			SwingUtilities.invokeLater(new Runnable()
			{
				@Override
				public void run()
				{
					l.onPause();
				}
			});
		}
	}

	@Override
	public void onSpaceCmd(Space space) {
		if (isListenersEnabled()) {
			fireSpaceListeners(space);
		}
	}
	
	public synchronized void fireSpaceListeners(final Space space) {
		assert isListenersEnabled();
		for (final SpaceCommandListener l : spaceListeners)
		{
			assert l != null;
			SwingUtilities.invokeLater(new Runnable()
			{
				@Override
				public void run()
				{
					l.onSpaceCmd(space);
				}
			});
		}
	}

	public UIInterpreterProxy(TraceData data) {
		assert data != null;
		
		this.data = data;
		
		this.pauseListeners = new ArrayList<PauseListener>();
		this.eventListeners = new ArrayList<EventCommandListener>();
		this.listenersEnabled = false;
		this.spaceListeners = new ArrayList<SpaceCommandListener>();
		
		setSynchronous(true);
	}

	@Override
	public synchronized void addEventListener(EventCommandListener eventListener) {
		assert eventListener != null;
		this.eventListeners.add(eventListener);
	}

	@Override
	public synchronized void addPauseListener(PauseListener pauseListener) {
		assert  pauseListener != null;
		this.pauseListeners.add(pauseListener);		
	}

	@Override
	public synchronized void addSpaceListener(SpaceCommandListener spaceListener) {
		assert spaceListener != null;
		this.spaceListeners.add(spaceListener);
	}

	@Override
	public synchronized void removeEventListener(EventCommandListener eventListener) {
		assert eventListener != null;
		this.eventListeners.remove(eventListener);
	}

	@Override
	public synchronized void removePauseListener(PauseListener pauseListener) {
		assert pauseListener != null;
		this.pauseListeners.remove(pauseListener);
	}

	@Override
	public synchronized void removeSpaceListener(SpaceCommandListener spaceListener) {
		assert spaceListener != null;
		this.spaceListeners.add(spaceListener);
	}

	@Override
	public int[] getEventCounts() {
		return getData().getEventCount();
	}

	@Override
	public Events getEvents() {
		return getData().getEvents();
	}

}
