/*
 * $Id
$
 * Copyright Richard Jones, University of Kent, 2005
 *
 * See the file "Kent_license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

package gcspy.vis.plugins;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * This class manages pluggable views. A pluggable view must provide
 * <ol>
 * <li>A short-form name
 * <li>A PluginListener class, <code><i>name</i>Listener</code>
 * <li>A PluginFrame class
 * </ol>
 * 
 * @author <a href="http://www/cs/kent.ac.uk">Richard Jones</a>
 */
public class PluginManager implements PluginManagerInterface {
	
	private static final Logger logger = Logger.getLogger(PluginManager.class.getName());

	/** The plugins directory is relative to the classpath */
	private static final String PLUGINS_DIR = "plugins/";

	/** Is a plugin names selected for viewing? **/
	private Map<String, Boolean> plugins;

	/** A class loader for the plugins */
	URLClassLoader pluginClassLoader;

	/**
	 * Create the Plugin Manager and load all plugins We expect the plugin to be
	 * in a jar file <code>plugins/<i>plugin</i>.jar</code>. We expect the
	 * plugin listener to be <code><i>Plugin</i>Listener</code>.
	 */
	public PluginManager() {
		plugins = new HashMap<String, Boolean>();
		List<URL> urls = new LinkedList<URL>();
		
		/*
		 * The following code is debug code which allows loading from directories containing class files,
		 * not jars.
		File basedir = new File("gcspy/vis/plugins/");
		File[] dirs = basedir.listFiles();
		for (File f : dirs) {
			if (f.isDirectory()) {
				try {
					urls.add(new URL("file", "localhost", f.getPath() + "/"));
					addToPlugins(f.getName());
				} catch (MalformedURLException e) {
					logger.log(Level.WARNING, "Could not add plugin directory {0}", f.getName());					
				}
			}
		}
		*/

		File dir = new File(PLUGINS_DIR);
		File[] files = dir.listFiles();
		if (files == null)
			return;
		for (File f : files) {
			String jarName = f.getName(); // foo.jar
			String p = basename(jarName, ".jar"); // foo
			if (p == null)
				continue;
			addToPlugins(p); // Foo
			try {
				urls.add(new URL("file", "localhost", PLUGINS_DIR + jarName));
			} catch (MalformedURLException e) {
				logger.log(Level.WARNING, "Could not add plugin jar " + PLUGINS_DIR
						+ jarName, e);
			}

		}

		URL[] urlArray = urls.toArray(new URL[urls.size()]);
		pluginClassLoader = new URLClassLoader(urlArray);
	}

	/**
	 * Drop a file name extension
	 * 
	 * @param name
	 *            the name of the file
	 * @param extension
	 *            the extension
	 * @return the name without the extension or null if the extension doesn't
	 *         match
	 */
	private static String basename(String name, String extension) {
		if (!name.endsWith(extension))
			return null;
		int end = name.length() - extension.length();
		return name.substring(0, end);
	}

	/**
	 * Add the name of a plugin to the plugins list. By default enable all plugins
	 * 
	 * @param p
	 *            Name of the plugin
	 */
	private void addToPlugins(String p) {
		StringBuffer sb = new StringBuffer(p.toLowerCase());
		sb.setCharAt(0, Character.toUpperCase(sb.charAt(0)));
		String pluginName = new String(sb);
		plugins.put(pluginName, true);
	}

	/**
	 * Get the names of all the plugins found
	 * 
	 * @return short-form names of all plugins
	 */
	public Set<String> getAllPlugins() {
		return plugins.keySet();
	}

	/**
	 * Get the listeners for all the plugins selected
	 * 
	 * @return the PluginListeners for all selected plugins
	 */
	public List<PluginListener> getPluginListeners() {
		List<PluginListener> listeners = new ArrayList<PluginListener>();
		for (String name : plugins.keySet()) {
			if (isActivePlugin(name)) {
				try {
					String className = "gcspy.vis.plugins."
							+ name.toLowerCase() + "." + name + "Listener";
					Class<?> clazz = pluginClassLoader.loadClass(className);
					PluginListener pluginListener = (PluginListener) clazz
							.newInstance();
					listeners.add(pluginListener);
				} catch (ClassNotFoundException e) {
					logger.log(Level.WARNING, "Cannot find " + name + "Listener class", e);
				} catch (InstantiationException e) {
					logger.log(Level.WARNING, "Cannot instantiate " + name + "Listener object", e);
				} catch (IllegalAccessException e) {
					logger.log(Level.WARNING, "Cannot access " + name + "Listener object", e);
				}
			}
		}
		return listeners;
	}

	/**
	 * Is a plugin selected
	 * 
	 * @param p
	 *            the plugin's name
	 * @return true if the plugin is selected
	 */
	public boolean isActivePlugin(String p) {
		return plugins.get(p);
	}

	/**
	 * Set a plugin's active state
	 * 
	 * @param plugin
	 *            The plugin's name
	 * @param selected
	 *            true if it is to be selected
	 */
	public void setActivePlugin(String plugin, boolean selected) {
		if (!plugins.containsKey(plugin))
			throw new PluginException("Cannot activate plugin " + plugin);
		plugins.put(plugin, selected);
	}
}
