/**
 ** $Id: Factory.java 28 2005-06-20 13:13:35Z rej $
 ** Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

package gcspy.vis.utils;

import gcspy.utils.ColorConstants;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableModel;

/**
 * Factory for GUI components
 * 
 * @author Tony Printezis
 * @author <a href="http://www/cs/kent.ac.uk">Richard Jones</a>
 */
public class Factory {

	/** *** Fonts **** */
	
	static private int DEFAULT_FONT_SIZE = new Font(null).getSize();
	static private int SMALL_FONT_SIZE = DEFAULT_FONT_SIZE - 2;

	static private Color TABLE_BACKGROUND = ColorConstants.OFF_WHITE;

	/** *** Buttons **** */

	/**
	 * Create a JButton
	 * 
	 * @param text
	 *            The label to use
	 * @return a new JButton
	 */
	static public JButton createButton(String text) {
		return createButton(text, true);
	}

	/**
	 * Create a JButton
	 * 
	 * @param text
	 *            The label to use
	 * @param al
	 *            An ActionListener for the button
	 * @return a new JButton
	 */
	static public JButton createButton(String text, ActionListener al) {
		return createButton(text, true, al);
	}

	/**
	 * Create a JButton
	 * 
	 * @param text
	 *            The label to use
	 * @param enabled
	 *            Is the button enabled?
	 * @return a new JButton
	 */
	static public JButton createButton(String text, boolean enabled) {
		return createButton(text, enabled, null);
	}

	/**
	 * Create a JButton
	 * 
	 * @param text
	 *            The label to use
	 * @param enabled
	 *            Is the button enabled?
	 * @param al
	 *            An ActionListener for the button
	 * @return a new JButton
	 */
	static public JButton createButton(String text, boolean enabled,
			ActionListener al) {
		return createButton(text, enabled, false, al);
	}

	/**
	 * Create a JButton
	 * 
	 * @param text
	 *            The label to use
	 * @param enabled
	 *            Is the button enabled?
	 * @param small
	 *            Is the button's font size small?
	 * @param al
	 *            An ActionListener for the button
	 * @return a new JButton
	 */
	static public JButton createButton(String text, boolean enabled,
			boolean small, ActionListener al) {
		return createButton(text, enabled, small, null, al);
	}

	/**
	 * Create a JButton
	 * 
	 * @param text
	 *            The label to use
	 * @param enabled
	 *            Is the button enabled?
	 * @param toolTipText
	 *            tool-tip text for the button
	 * @param al
	 *            An ActionListener for the button
	 * @return a new JButton
	 */
	static public JButton createButton(String text, boolean enabled,
			String toolTipText, ActionListener al) {
		return createButton(text, enabled, false, toolTipText, al);
	}

	/**
	 * Create a JButton
	 * 
	 * @param text
	 *            The label to use
	 * @param enabled
	 *            Is the button enabled?
	 * @param small
	 *            Is the button's font size small?
	 * @param toolTipText
	 *            tool-tip text for the button
	 * @param al
	 *            An ActionListener for the button
	 * @return a new JButton
	 */
	static public JButton createButton(String text, boolean enabled,
			boolean small, String toolTipText, ActionListener al) {
		JButton button = new JButton(text);
		button.setEnabled(enabled);
		if (small) {
			Insets margin = (Insets) button.getMargin().clone();
			margin.left = margin.top;
			margin.right = margin.top;
			margin.top = margin.top / 2;
			margin.bottom = margin.top;
			button.setMargin(margin);
			button.setFont(button.getFont().deriveFont((float) SMALL_FONT_SIZE));
		}
		if (toolTipText != null)
			button.setToolTipText(toolTipText);
		if (al != null)
			button.addActionListener(al);
		return button;
	}

	/**
	 * Create a JButton with an icon
	 * 
	 * @param icon
	 *            The icon
	 * @return a new JButton
	 */
	static public JButton createIconButton(Icon icon) {
		return createIconButton(icon, true, null);
	}

	/**
	 * Create a JButton with an icon
	 * 
	 * @param icon
	 *            The icon
	 * @param enable
	 *            Is the button enabled?
	 * @param al
	 *            ActionListener for the button
	 * @return a new JButton
	 */
	static public JButton createIconButton(Icon icon, boolean enable,
			ActionListener al) {
		return createIconButton(icon, enable, null, al);
	}

	/**
	 * Create a JButton with an icon
	 * 
	 * @param icon
	 *            The icon
	 * @param enable
	 *            Is the button enabled?
	 * @param toolTipText
	 *            Tool-tip text for the button
	 * @param al
	 *            ActionListener for the button
	 * @return a new JButton
	 */
	static public JButton createIconButton(Icon icon, boolean enable,
			String toolTipText, ActionListener al) {
		JButton button = new JButton(icon);

		Insets margin = (Insets) button.getMargin().clone();
		margin.left = margin.top;
		margin.right = margin.top;
		margin.bottom = margin.top;
		button.setMargin(margin);
		button.setEnabled(enable);
		if (toolTipText != null)
			button.setToolTipText(toolTipText);
		if (al != null)
			button.addActionListener(al);

		return button;
	}

	/** *** CheckBoxes **** */

	/**
	 * Create a new JCheckBoxMenuItem
	 * 
	 * @param text
	 *            The text for the menu item
	 * @return a new JCheckBoxMenuItem
	 */
	static public JCheckBoxMenuItem createCheckBoxMenuItem(String text) {
		return createCheckBoxMenuItem(text, true);
	}

	/**
	 * Create a new JCheckBoxMenuItem
	 * 
	 * @param text
	 *            The text for the menu item
	 * @param state
	 *            The state of the check-box
	 * @return a new JCheckBoxMenuItem
	 */
	static public JCheckBoxMenuItem createCheckBoxMenuItem(String text,
			boolean state) {
		return createCheckBoxMenuItem(text, state, null);
	}

	/**
	 * Create a new JCheckBoxMenuItem
	 * 
	 * @param text
	 *            The text for the menu item
	 * @param state
	 *            The state of the check-box
	 * @param al
	 *            An ActionListener for the item
	 * @return a new JCheckBoxMenuItem
	 */
	static public JCheckBoxMenuItem createCheckBoxMenuItem(String text,
			boolean state, ActionListener al) {
		JCheckBoxMenuItem menuItem = new JCheckBoxMenuItem(text);
		menuItem.setState(state);
		if (al != null)
			menuItem.addActionListener(al);
		return menuItem;
	}

	/**
	 * Create a new labelled JCheckBox
	 * 
	 * @return return a new JCheckBox
	 */
	static public JCheckBox createCheckBox() {
		return createCheckBox(null, true, null);
	}

	/**
	 * Create a new labelled JCheckBox
	 * 
	 * @param selected
	 *            Is the box selected?
	 * @return return a new JCheckBox
	 */
	static public JCheckBox createCheckBox(boolean selected) {
		return createCheckBox(null, selected, null);
	}

	/**
	 * Create a new labelled JCheckBox
	 * 
	 * @param text
	 *            The text to label the check-box
	 * @return return a new JCheckBox
	 */
	static public JCheckBox createCheckBox(String text) {
		return createCheckBox(text, true, null);
	}

	/**
	 * Create a new labelled JCheckBox
	 * 
	 * @param text
	 *            The text to label the check-box
	 * @param selected
	 *            Is the box selected?
	 * @return return a new JCheckBox
	 */
	static public JCheckBox createCheckBox(String text, boolean selected) {
		return createCheckBox(text, selected, null);
	}

	/**
	 * Create a new labelled JCheckBox
	 * 
	 * @param text
	 *            The text to label the check-box
	 * @param selected
	 *            Is the box selected?
	 * @param al
	 *            An ActionListener for the check-box
	 * @return return a new JCheckBox
	 */
	static public JCheckBox createCheckBox(String text, boolean selected,
			ActionListener al) {
		return createCheckBox(text, true, selected, al);
	}

	/**
	 * Create a new labelled JCheckBox
	 * 
	 * @param text
	 *            The text to label the check-box
	 * @param enabled
	 *            Is the check-box enabled?
	 * @param selected
	 *            Is the box selected?
	 * @param al
	 *            An ActionListener for the check-box
	 * @return return a new JCheckBox
	 */
	static public JCheckBox createCheckBox(String text, boolean enabled,
			boolean selected, ActionListener al) {
		JCheckBox checkBox;
		if (text != null)
			checkBox = new JCheckBox(text);
		else
			checkBox = new JCheckBox();
		checkBox.setEnabled(enabled);
		checkBox.setSelected(selected);
		if (al != null)
			checkBox.addActionListener(al);
		return checkBox;
	}

	/** *** ComboBoxes **** */

	/**
	 * Create a new combo box
	 * 
	 * @return the combo box
	 */
	static public JComboBox createComboBox() {
		return createComboBox(null);
	}

	/**
	 * Create a new combo box
	 * 
	 * @param al
	 *            An ActionListener
	 * @return the combo box
	 */
	static public JComboBox createComboBox(ActionListener al) {
		JComboBox comboBox = new JComboBox();
		if (al != null)
			comboBox.addActionListener(al);
		return comboBox;
	}

	/** *** Labels **** */

	/**
	 * Create a JLabel
	 * 
	 * @param text
	 *            Text for the label
	 * @param fontStyle
	 *            Font style, see {@link Font#getStyle()}.
	 * @param alignment
	 *            Horizontal alignment
	 * @return a new JLabel
	 */
	static public JLabel createLabel(String text, int fontStyle) {
		return createLabel(text, fontStyle, JLabel.LEADING);
	}

	/**
	 * Create a JLabel.
	 * 
	 * @param text
	 *            Text for the label.
	 * @param fontStyle
	 *            Font style, see {@link Font#getStyle()}.
	 * @param alignment
	 *            Horizontal alignment, see
	 *            {@link JLabel#getHorizontalAlignment()}.
	 * @return a new JLabel
	 */
	static public JLabel createLabel(String text, int fontStyle, int alignment) {
		JLabel label = new JLabel(text, alignment);
		if (fontStyle != Font.PLAIN)
			label.setFont(label.getFont().deriveFont(fontStyle));
		return label;
	}

	/**
	 * Create a JLabel
	 * 
	 * @param text
	 *            Text for the label
	 * @return a new JLabel
	 */
	static public JLabel createLabel(String text) {
		return new JLabel(text);
	}

	/** *** Panels **** */

	/**
	 * Create an empty panel
	 * 
	 * @param width
	 *            The width of the panel
	 * @param height
	 *            The height of the panel [Why not use a box Box? ]
	 */
	static public JPanel createEmptyPanel(int width, int height) {
		JPanel panel = new JPanel();
		panel.setBorder(BorderFactory.createEmptyBorder(height, width, 0, 0));
		return panel;
	}

	/* Panel with Empty Border */

	/**
	 * Create an empty panel with a border
	 * 
	 * @param border
	 *            the width of the border
	 * @return the panel
	 */
	static public JPanel createEmptyPanel(int border) {
		return createEmptyPanel(border, null);
	}

	/**
	 * Place a component in a new panel with a border
	 * 
	 * @param border
	 *            the width of the border
	 * @param comp
	 *            the component
	 * @return the panel
	 */
	static public JPanel createEmptyPanel(int border, Component comp) {
		JPanel panel = new JPanel();
		panel.setBorder(BorderFactory.createEmptyBorder(border, border, border,
				border));
		if (comp != null)
			panel.add(comp);
		return panel;
	}

	/* Panel with Title Border */

	/**
	 * Create a new panel with a title
	 * 
	 * @param title
	 *            the title
	 * @return the new panel
	 */
	static public JPanel createTitlePanel(String title) {
		return createTitlePanel(title, null);
	}

	/**
	 * Place a component in a panel with a title
	 * 
	 * @param title
	 *            The title
	 * @param comp
	 *            The component
	 * @return the new panel
	 */
	static public JPanel createTitlePanel(String title, Component comp) {
		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout());
		panel.setBorder(createTitledBorder(title));
		if (comp != null)
			panel.add(comp, BorderLayout.CENTER);
		return panel;
	}

	/* Panel with Border Layout */

	/**
	 * Create a new panel with a border layout
	 * 
	 * @return a new panel with a border layout
	 */
	static public JPanel createBorderPanel() {
		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout());
		return panel;
	}

	/* Panel with Flow Layout */

	/**
	 * Create a new panel with a flow layout
	 * 
	 * @param align
	 *            The alignment
	 * @return the new panel
	 */
	static public JPanel createFlowPanel(int align) {
		JPanel panel = new JPanel();
		panel.setLayout(new FlowLayout(align));
		return panel;
	}

	/* Panel with Grid Layout */

	/**
	 * Create a new panel with a title, using grid layout
	 * 
	 * @param rows
	 *            The number of rows
	 * @param columns
	 *            The number of columns
	 * @return the new panel
	 */
	static public JPanel createGridPanel(int rows, int columns) {
		JPanel panel = new JPanel();
		panel.setLayout(new GridLayout(rows, columns));
		return panel;
	}
	
	static public JPanel createGridBagPanel() {
		JPanel panel = new JPanel();
		panel.setLayout(new GridBagLayout());
		return panel;		
	}

	/**
	 * Create a new panel with a title, using grid layout
	 * 
	 * @param rows
	 *            The number of rows
	 * @param columns
	 *            The number of columns
	 * @param title
	 *            the title of the panel
	 * @return the new panel
	 */
	static public JPanel createGridPanel(int rows, int columns, String title) {
		JPanel panel = createGridPanel(rows, columns);
		if (title != null)
			panel.setBorder(createTitledBorder(title));
		else
			panel.setBorder(createPlainBorder());
		return panel;
	}

	/** *** Tables **** */

	/**
	 * Create a new table
	 * 
	 * @param model
	 *            The table's model
	 * @return the new table
	 */
	static public JTable createTable(TableModel model) {
		return createTable(model, false, true);
	}

	/**
	 * Create a new table
	 * 
	 * @param model
	 *            The table's model
	 * @param resizable
	 *            Treu if the user can resize/reorder columns?
	 * @param auto
	 *            True if columns are automatically created from the model
	 * @return the new table
	 */
	static public JTable createTable(TableModel model, boolean resizable,
			boolean auto) {
		JTable table = new JTable(model);
		table.setAutoCreateColumnsFromModel(auto);

		table.setRowSelectionAllowed(false);
		table.setColumnSelectionAllowed(false);
		table.setBackground(TABLE_BACKGROUND);

		JTableHeader header = table.getTableHeader();
		header.setReorderingAllowed(resizable);
		header.setResizingAllowed(resizable);

		return table;
	}

	/** *** Text Areas **** */

	static public JTextArea createTextArea(boolean editable, boolean monospace) {
		return createTextArea(0, 0, editable, monospace);
	}

	static public JTextArea createTextArea(int rows, int columns, boolean editable) {
		return createTextArea(rows, columns, editable, false);
	}

	/**
	 * Create a new text area
	 * 
	 * @param rows
	 *            The number of rows
	 * @param columns
	 *            The number of columns
	 * @param editable
	 *            True if the text area is editable
	 * @param monospaced
	 *            True if the font is monospaced
	 * @return a new text area
	 */
	static public JTextArea createTextArea(int rows, int columns, boolean editable, boolean monospaced) {
		JTextArea textArea = new JTextArea(rows, columns);
		textArea.setEditable(editable);
		if (monospaced) {
			Font original = textArea.getFont();
			textArea.setFont(new Font(Font.MONOSPACED, original.getStyle(), original.getSize()));
		}
		return textArea;
	}

	/** *** Text Fields **** */

	/**
	 * Create a new text field
	 * 
	 * @param columns
	 *            The columns in the field
	 * @param editable
	 *            True if the field is editable
	 * @return a new text field
	 */
	static public JTextField createTextField(int columns, boolean editable) {
		return createTextField(null, columns, editable);
	}

	/**
	 * Create a new text field
	 * 
	 * @param text
	 *            The text to place in the field
	 * @param columns
	 *            The columns in the field
	 * @param editable
	 *            True if the field is editable
	 * @return a new text field
	 */
	static public JTextField createTextField(String text, int columns,
			boolean editable) {
		JTextField textField = new JTextField(columns);
		textField.setEditable(editable);
		if (text != null)
			textField.setText(text);
		return textField;
	}

	/** *** Titled Borders **** */

	/**
	 * Create a new plain border
	 * 
	 * @return a new plain area
	 */
	static public TitledBorder createPlainBorder() {
		return createTitledBorder(null);
	}

	/**
	 * Create a new titled border
	 * 
	 * @param title
	 *            The title to use
	 * @return a new titled area
	 */
	static public TitledBorder createTitledBorder(String title) {
        return BorderFactory.createTitledBorder(title);
	}

}
